#!/usr/local/bin/perl

# nagios: +epn

=pod

=head1 NAME

check_bluecoat_cpu.pl - Check CPU pressure on a Bluecoat device.

=head1 DESCRIPTION

The SNMP agent on Bluecoat devices allows us to query it to see what
thresholds have been set by the Bluecoat administrator.  For this reason
we do not need to have the user specify warning nor critical thresholds.

The script will return perfdata for CPU utilization for all Bluecoat 
devices; for SGOS 5.x and later this script will also return idle and
busy % CPU utilization.

=cut

sub check_bluecoat_cpu {

    use strict;
    use FindBin;
    use lib "$FindBin::Bin/lib";

    use Net::SNMP;
    use Nagios::Plugin::SNMP;

    my $LABEL = 'BLUECOAT-CPU';

    my $plugin = Nagios::Plugin::SNMP->new(
        'shortname' => $LABEL,
        'usage' => 'USAGE: %s'
    );

    $plugin->getopts;

    my $DEBUG = $plugin->opts->get('snmp-debug');

    #  Return from current state will be one of these, each has
    #  a value that matches the index position in the array below.
    my @states = qw(ok low-warning warning high-warning
                    low-critical critical high-critical);

    my %oids = qw(
        sgProxyCpuBusyPerCent.0   .1.3.6.1.4.1.3417.2.11.2.1.7.0
        sgProxyCpuIdlePerCent.0   .1.3.6.1.4.1.3417.2.11.2.1.8.0
        cpuUtilizationValue.0     .1.3.6.1.4.1.3417.2.8.1.3.0
        cpuCurrentState.0         .1.3.6.1.4.1.3417.2.8.1.9.0
        cpuWarningThreshold.0     .1.3.6.1.4.1.3417.2.8.1.4.0
        cpuCriticalThreshold.0    .1.3.6.1.4.1.3417.2.8.1.6.0
    );

    my $result = $plugin->get(values %oids);

    #  Close and destroy session
    $plugin->close();

    my @perf_data;

    my %cpu;

    $cpu{'busy'} = $result->{$oids{'sgProxyCpuBusyPerCent.0'}};
    $cpu{'idle'} = $result->{$oids{'sgProxyCpuIdlePerCent.0'}};
    $cpu{'util'} = $result->{$oids{'cpuUtilizationValue.0'}};
    $cpu{'state'} = $result->{$oids{'cpuCurrentState.0'}};
    $cpu{'warn'} = $result->{$oids{'cpuWarningThreshold.0'}};
    $cpu{'crit'} = $result->{$oids{'cpuCriticalThreshold.0'}};

    #  Not supported on SGOS < 5.x
    $cpu{'busy'} = 0 if $cpu{'busy'} eq 'noSuchObject';
    $cpu{'idle'} = 0 if $cpu{'idle'} eq 'noSuchObject';

    my $level = OK;

    my $output = "$LABEL ";

    if ($cpu{'state'} == 0) {
        $output .= "OK - Utilization $cpu{'util'}%";
    } elsif (($cpu{'state'} > 0) && ($cpu{'state'} < 6)) {
        $output .= "WARNING  - Utilization $cpu{'util'}% >= $cpu{'warn'}%";
        $level = WARNING;
    } else {
        $output .= "CRITICAL - Utilization $cpu{'util'}% >= $cpu{'crit'}%";
        $level = CRITICAL;
    }

    print "$output, state $states[$cpu{'state'}] ",
          "| 'busy'=$cpu{'busy'}%;;;0;100 ",
          "'idle'=$cpu{'idle'}%;;;0;100 ",
          "'util'=$cpu{'util'}%;$cpu{'warn'};$cpu{'crit'};0;100 ",
          "'state'=$cpu{'state'};1;6;0;6\n";

    return $level;

    sub debug {

        return unless $DEBUG == 1;

        my $msg = shift;

        print STDERR scalar(localtime()) . ": $msg\n";

    }

}

exit check_bluecoat_cpu();
