#!/usr/local/bin/perl

# nagios: +epn

=pod

=head1 NAME

check_bluecoat_mem.pl - Check Bluecoat memory pressure

=head1 DESCRIPTION

This script will check memory pressure on a Bluecoat device.  The Bluecoat
SNMP agent also exposes the thresholds the Bluecoat administrator has set
on the device, which means we do not need to have the user provide
warning or critical thresholds.

The script will return memory pressure % as perfdata; for Bluecoats running
SGOS version 5.x or higher, the script will also return cached, sys, and
avail memory as perfdata for trending purposes.

=cut

sub check_bluecoat_mem {

    use strict;

    use FindBin;
    use lib "$FindBin::Bin/lib";

    use Nagios::Plugin::SNMP;

    my $LABEL = 'BLUECOAT MEM';

    my $plugin = Nagios::Plugin::SNMP->new(
        'shortname' => $LABEL,
        'usage' => 'USAGE: %s'
    );

    $plugin->getopts;

    my $DEBUG = $plugin->opts->get('snmp-debug');

    #  Return from current state will be one of these:
    my @states = qw(ok low-warning warning high-warning
                    low-critical critical high-critical);

    my %oids = qw(
        sgProxyMemAvailable.0  .1.3.6.1.4.1.3417.2.11.2.3.1.0
        sgProxyMemCacheUsage.0 .1.3.6.1.4.1.3417.2.11.2.3.2.0
        sgProxyMemSysUsage.0   .1.3.6.1.4.1.3417.2.11.2.3.3.0
        memPressureValue.0     .1.3.6.1.4.1.3417.2.8.2.3.0
        memCurrentState.0      .1.3.6.1.4.1.3417.2.8.2.9.0
        memWarningThreshold.0  .1.3.6.1.4.1.3417.2.8.2.4.0
        memCriticalThreshold.0 .1.3.6.1.4.1.3417.2.8.2.7.0
    );

    #  Get the mem values
    my $result = $plugin->get(values %oids);

    #  Close and destroy session
    $plugin->close();

    my @perf_data;

    my %mem;

    $mem{'avail'} = $result->{$oids{'sgProxyMemAvailable.0'}};
    $mem{'cache'} = $result->{$oids{'sgProxyMemCacheUsage.0'}};
    $mem{'sys'} = $result->{$oids{'sgProxyMemSysUsage.0'}};
    $mem{'pressure'} = $result->{$oids{'memPressureValue.0'}};

    $mem{'state'} = $result->{$oids{'memCurrentState.0'}};
    $mem{'warn'} = $result->{$oids{'memWarningThreshold.0'}};
    $mem{'crit'} = $result->{$oids{'memCriticalThreshold.0'}};

    $mem{'avail'} = 0 if $mem{'avail'} eq 'noSuchObject';
    $mem{'cache'} = 0 if $mem{'cache'} eq 'noSuchObject';
    $mem{'sys'} = 0 if $mem{'sys'} eq 'noSuchObject';
    $mem{'pressure'} = '' if $mem{'pressure'} eq 'noSuchObject';

    my $level = OK;
    my $output = "$LABEL ";

    if ($mem{'state'} == 0) {
        $output .= "OK - Utilization $mem{'pressure'}%";
    } elsif (($mem{'state'} > 0) && ($mem{'state'} < 6)) {
        $output .= "WARNING  - Utilization $mem{'pressure'}% >= $mem{'warn'}%";
        $level = WARNING;
    } else {
        $output .= "CRITICAL - Utilization $mem{'pressure'}% >= $mem{'crit'}%";
        $level = CRITICAL;
    }

    print "$output, state $states[$mem{'state'}] ",
          "| 'avail'=$mem{'avail'}b;;;; ",
          "'cache'=$mem{'cache'}b;;;; ",
          "'sys'=$mem{'sys'}b;;;; ",
          "'pressure'=$mem{'pressure'}%;$mem{'warn'};$mem{'crit'};0;100 ",
          "'state'=$mem{'state'};1;6;0;6\n";

    return $level;

    sub debug {

        return unless $DEBUG == 1;

        my $msg = shift;

        print STDERR scalar(localtime()) . ": $msg\n";

    }

}

exit check_bluecoat_mem();
