package WWW::UltimateDomains;

$WWW::UltimateDomains::VERSION = 
    '#  $Id: UltimateDomains.pm 17 2008-01-06 05:28:48Z max $';

use strict;
use warnings;

use WWW::Mechanize;
use Time::HiRes qw( gettimeofday tv_interval );

=pod

=head1 SYNOPSIS

 WWW::UltimateDomains - utilities for automation of tests of the
 ultimatedomains.com site

=cut

=pod

=item new('Timeout' => 60)

  Instantiate WWW::UltimateDomains class.  Parameters:
  SubDomain - Sub-domain of ultimatedomains.com to test against [optional]
  Timeout - Maximum seconds any operation should take
  UserAgent - User-Agent header value [optional]

  my $ud = WWW::UltimateDomains->new(
     'Timeout' => 60,
     #  The HTTP User-agent to use (optional)
     'UserAgent' => "Query Agent",  
  );

  my ($results, $time) = $ud->keyword_search('cards');

=cut

sub new {

    my $class = shift;
    my $self = {
        'Timeout' => undef,
        'SubDomain' => 'www',
        'UserAgent' => 'UltimateDomainsTester/1.0'
    };

    my %args = @_;

    if (scalar(@_) == 0) {

        die <<EOF;

Missing options to new:
    Timeout    - Maximum timeout in seconds for any called method [required]
    SubDomain - sub-domain to test against (e.g. www, dev, beta) [optional]
    UserAgent  - Textual string to use for User-Agent header [optional]
Defaults:
    SubDomain: ${self}->{SubDomain}
    UserAgent: ${self}->{Useragent}
EOF

    }

    $self->{'Timeout'} = $args{'Timeout'} ||
        die "Missing required parameter `Timeout'!";

    $self->{'SubDomain'} = $args{'SubDomain'} if defined($args{'SubDomain'});
    $self->{'UserAgent'} = $args{'UserAgent'} if defined($args{'UserAgent'});

    $self->{'_base_url'} = "http://$args{'SubDomain'}.ultimatedomains.com";

    # XXX - need error handling at some point
    $self->{'_agent'} = WWW::Mechanize->new('agent' => $self->{'UserAgent'});
    $self->{'_agent'}->timeout($self->{'Timeout'});

    return bless $self, $class;

}

=item keyword_search

 Performs a HTTP request on keyword_search.php with a given search term.  
 It returns the number domains found by keyword_search.php, the time 
 it took to perform the HTTP request, and an error message if a catchable
 error occurred.

 Usage:

  my $ud = WWW::UltimateDomains->new('Timeout' => 60);
  my ($found, $time, $error) = $ud->keyword_search("search_term");

  die "Search failed: $error!" if defined $error;

=cut

sub keyword_search {

    my $self = shift;
    my $term = shift || die "Missing term to search for!";

    my $url = $self->{'_base_url'} . '/keyword_search.php';
    my $agent  = $self->{'_agent'};

    #  Encode non-alphanumeric characters in the search term.

    $term =~ s/([^A-Za-z0-9 ])/sprintf("%%%02X", ord($1))/seg;
    $term =~ s/ /+/g;

    #  Do the page fetch and time it
    my $start_time_ref = [ gettimeofday ];

    eval {
        $agent->get("${url}?search=${term}");
    };

    my $err = $@;

    my $search_time = tv_interval($start_time_ref);

    #  Parse the content and look for the "Total Domains" string
    my $domains_found = 'No';

    my $content = $agent->content();

    my $err_msg = "";

    if ($err ne '') {
        $err_msg = "Search failed: $@";
    } elsif ($content =~ m/Total Domains\D+(\d+)/i) {
        $domains_found = $1;
    } elsif ( $content =~ m/No domains were found/ ) {
        $domains_found = 0;
    } elsif ( $content =~ m/500 read timeout/ ) {
        $err_msg = "Error: search timed out (${search_time}s > " .
            $self->{'Timeout'} . "s)";
    } else {
        $err_msg = "Expected 'Total Domains\\D+\\d+', received $content";
    }

    #  Return number of domains found and search time
    return ($domains_found, $search_time, $err_msg);

}

1;
